# Run files

These tests examine the files generated by a run.

We use the `batch` sample project for our tests.

    >>> cd(sample("projects", "batch"))

Isolate our runs using a new Guild home.

    >>> set_guild_home(mkdtemp())

We use an environment for runs that reverts the disabling of pip
freeze.

    >>> PipFreezeEnabled = Env({"NO_PIP_FREEZE": "0"})

## Basic Python based run

The script `say.py` prints a message.

    >>> with PipFreezeEnabled:
    ...     run("guild run say.py msg=hi -y")
    hi

    >>> run("guild runs -s")
    [1]  say.py  completed  loud=no msg=hi

Show the run files.

    >>> run("guild ls --all -n")  # doctest: +REPORT_UDIFF
    .guild/
    .guild/attrs/
    .guild/attrs/cmd
    .guild/attrs/deps
    .guild/attrs/env
    .guild/attrs/exit_status
    .guild/attrs/flags
    .guild/attrs/host
    .guild/attrs/id
    .guild/attrs/initialized
    .guild/attrs/label
    .guild/attrs/op
    .guild/attrs/pip_freeze
    .guild/attrs/platform
    .guild/attrs/plugins
    .guild/attrs/random_seed
    .guild/attrs/run_params
    .guild/attrs/sourcecode_digest
    .guild/attrs/started
    .guild/attrs/stopped
    .guild/attrs/user
    .guild/attrs/user_flags
    .guild/manifest
    .guild/opref
    .guild/output
    .guild/output.index
    add.py
    batch-2.csv
    batch.csv
    batch.json
    batch.unknown
    batch.yaml
    batch.yml
    error.py
    guild.yml
    invalid-data.json
    invalid-data.yml
    invalid-item.json
    invalid-item.yaml
    say.py
    single-run-2.json
    single-run.csv
    single-run.json
    tune.py

Note that run source code files are copied to the run directory
root. Prior to 0.9, source code was copied to `.guild/sourcecode`.

Here are some generated files:

    >>> run("guild cat -p .guild/attrs/cmd")
    - ...
    - -um
    - guild.op_main
    - say
    - --msg
    - hi

    >>> run("guild cat -p .guild/attrs/deps")
    {}

    >>> run("guild cat -p .guild/attrs/exit_status")
    0

    >>> run("guild cat -p .guild/attrs/flags")
    loud: false
    msg: hi

    >>> run("guild cat -p .guild/opref")
    script:.../samples/projects/batch... '' '' say.py

    >>> run("guild cat -p .guild/output")
    hi

    >>> run("guild cat -p .guild/attrs/platform")
    architecture: ...
    cpus: ...
    processor: ...
    python_version: ...
    uname: ...

## Batch runs

Batch runs are normal Guild runs and generate the same list of core
files. They also contain a `proto` run, which serves to generate batch
trials.

Let's generate a batch using a flag value list.

    >>> with PipFreezeEnabled:
    ...     run("guild run say.py msg=[ho] --keep-batch -y")
    INFO: [guild] Running trial ...: say.py (loud=no, msg=ho)
    ho

The batch generates two runs, one for the batch and the other for the
trial (the third run is from our previous test):

    >>> run("guild runs -s")
    [1]  say.py   completed  loud=no msg=ho
    [2]  say.py+  completed
    [3]  say.py   completed  loud=no msg=hi

List the files for the batch (run 2). We ignore linked run IDs using a
regex.

    >>> run("guild ls -n --all 2", ignore=r"^[a-f0-9]{32}")  # doctest: +REPORT_UDIFF
    .guild/
    .guild/attrs/
    .guild/attrs/cmd
    .guild/attrs/deps
    .guild/attrs/env
    .guild/attrs/exit_status
    .guild/attrs/flags
    .guild/attrs/host
    .guild/attrs/id
    .guild/attrs/initialized
    .guild/attrs/objective
    .guild/attrs/op
    .guild/attrs/platform
    .guild/attrs/plugins
    .guild/attrs/random_seed
    .guild/attrs/run_params
    .guild/attrs/sourcecode_digest
    .guild/attrs/started
    .guild/attrs/stopped
    .guild/attrs/user
    .guild/attrs/user_flags
    .guild/manifest
    .guild/opref
    .guild/output
    .guild/output.index
    .guild/proto/
    .guild/proto/.guild/
    .guild/proto/.guild/PENDING
    .guild/proto/.guild/attrs/
    .guild/proto/.guild/attrs/cmd
    .guild/proto/.guild/attrs/flags
    .guild/proto/.guild/attrs/host
    .guild/proto/.guild/attrs/id
    .guild/proto/.guild/attrs/initialized
    .guild/proto/.guild/attrs/label
    .guild/proto/.guild/attrs/op
    .guild/proto/.guild/attrs/platform
    .guild/proto/.guild/attrs/plugins
    .guild/proto/.guild/attrs/random_seed
    .guild/proto/.guild/attrs/run_params
    .guild/proto/.guild/attrs/sourcecode_digest
    .guild/proto/.guild/attrs/user
    .guild/proto/.guild/attrs/user_flags
    .guild/proto/.guild/manifest
    .guild/proto/.guild/opref
    .guild/proto/add.py
    .guild/proto/batch-2.csv
    .guild/proto/batch.csv
    .guild/proto/batch.json
    .guild/proto/batch.unknown
    .guild/proto/batch.yaml
    .guild/proto/batch.yml
    .guild/proto/error.py
    .guild/proto/guild.yml
    .guild/proto/invalid-data.json
    .guild/proto/invalid-data.yml
    .guild/proto/invalid-item.json
    .guild/proto/invalid-item.yaml
    .guild/proto/say.py
    .guild/proto/single-run-2.json
    .guild/proto/single-run.csv
    .guild/proto/single-run.json
    .guild/proto/tune.py

Note the additional `proto` directory. This contains the prototype for
the `say.py` operation, which is used by the batch operation to
generate each specific trial op.
